﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.MainForm;
using HeuristicLab.Optimization.Views;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Problems.DataAnalysis.Views {
  [View("DataAnalysisProblem View")]
  [Content(typeof(IDataAnalysisProblem), true)]
  public partial class DataAnalysisProblemView : ProblemView {
    public DataAnalysisProblemView() {
      InitializeComponent();
    }

    public new IDataAnalysisProblem Content {
      get { return (IDataAnalysisProblem)base.Content; }
      set { base.Content = value; }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      importButton.Enabled = !Locked && !ReadOnly && Content != null;
    }

    private void ImportButton_Click(object sender, System.EventArgs e) {
      if (openFileDialog.ShowDialog(this) == DialogResult.OK) {
        try {
          Content.ImportProblemDataFromFile(openFileDialog.FileName);
        }
        catch (Exception ex) {
          ErrorHandling.ShowErrorDialog(this, ex);
        }
      }
    }

    private void loadButton_Click(object sender, EventArgs e) {
      if (benchmarkComboBox.SelectedItem != null)
        Content.ProblemData = ((IDataAnalysisBenchmarkProblemDataGenerator)benchmarkComboBox.SelectedItem).GenerateProblemData();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      benchmarkComboBox.Items.Clear();
      benchmarkComboBox.Items.AddRange(GetBenchmarkProblemDataGenerators().OrderBy(b => b.Name, new NaturalStringComparer()).ToArray());
      if (benchmarkComboBox.Items.Count > 0)
        benchmarkComboBox.SelectedIndex = 0;
    }

    protected IEnumerable<IDataAnalysisBenchmarkProblemDataGenerator> GetBenchmarkProblemDataGenerators() {
      if (Content is IRegressionProblem)
        return ApplicationManager.Manager.GetInstances<IRegressionBenchmarkProblemDataGenerator>();
      else if (Content is IClassificationProblem)
        return ApplicationManager.Manager.GetInstances<IClassificationBenchmarkProblemDataGenerator>();
      else if (Content is ITimeSeriesPrognosisProblem)
        return ApplicationManager.Manager.GetInstances<ITimeSeriesBenchmarkProblemDataGenerator>();
      else if (Content is IClusteringProblem)
        return ApplicationManager.Manager.GetInstances<IClusteringBenchmarkProblemDataGenerator>();
      return new List<IDataAnalysisBenchmarkProblemDataGenerator>();
    }
  }
}
